/***************************************************************************
                          table.cpp - Table widget
                             -------------------
    copyright            : (C) 2004      Michal Rudolf <mrudolf@kdewebdev.org>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/* KDE INCLUDES */

/* QT INCLUDES */
#include <qstring.h>
#include <qwidget.h>
#include <qstringlist.h>

/* OTHER INCLUDES */
#include <specials.h>

#include "kom-fsmi.h"

#define FSMI_PLUGIN_MAGIC 1250

namespace TableFunction
{
	enum { start = FSMI_PLUGIN_MAGIC, adjustColumn, setColumnStretchable };
}

FF_Table::FF_Table(QWidget *a_parent, const char *a_name)
  : QTable(a_parent, a_name), KommanderWidget(this)
{
  QStringList states;
  states << "default";
  setStates(states);
  setDisplayStates(states);

  SpecialInformation::setDefaultGroup(Group::DCOP);
  SpecialInformation::insert(TableFunction::adjustColumn, "adjustColumn(QString widget, int col)",  "Resizes column col so that the column width is wide enough to display the widest item the column contains.");
  //SpecialInformation::addDCOP(TableFunction::adjustColumn, "void");
  
  SpecialInformation::insert(TableFunction::setColumnStretchable, "setColumnStretchable(QString widget, int col, bool stretch)",  "If stretch is TRUE, column col is set to be stretchable; otherwise column col is set to be unstretchable.\nIf the table widget's width decreases or increases stretchable columns will grow narrower or wider to fit the space available as completely as possible. The user cannot manually resize stretchable columns.");
  //SpecialInformation::addDCOP(TableFunction::setColumnStretchable, "void");
  
  /* FSMI specific defaults */

  setSorting(true);
  setSortWholeRows(true);
  setSelectionMode(SingleRow);
  setReadOnly(true);
  setFocusStyle(FollowStyle); /*- Looks nicer but has a bug if sorting is activated */
}

FF_Table::~FF_Table()
{
}

QString FF_Table::currentState() const
{
  return QString("default");
}

bool FF_Table::isKommanderWidget() const
{
  return true;
}

QStringList FF_Table::associatedText() const
{
  return KommanderWidget::associatedText();
}

void FF_Table::setAssociatedText(const QStringList& a_at)
{
  KommanderWidget::setAssociatedText(a_at);
}

void FF_Table::setPopulationText(const QString& a_text)
{
  KommanderWidget::setPopulationText(a_text);
}

QString FF_Table::populationText() const
{
  return KommanderWidget::populationText();
}

void FF_Table::populate()
{
  setWidgetText(KommanderWidget::evalAssociatedText( populationText()));
}

void FF_Table::setWidgetText(const QString&)
{
}

bool FF_Table::isFunctionSupported(int f)
{
  return f == DCOP::currentColumn || f == DCOP::currentRow || f == DCOP::insertColumn || 
      f == DCOP::insertRow || f == DCOP::cellText || f == DCOP::setCellText ||
      f == DCOP::removeRow || f == DCOP::removeColumn || f == DCOP::setColumnCaption ||
      f == DCOP::setRowCaption || f == DCOP::text || f == DCOP::setText || f >= TableFunction::start;
}

QString FF_Table::handleDCOP(int function, const QStringList& args)
{
  switch (function) 
  {
    case DCOP::cellText:
      return text(args[0].toInt(), args[1].toInt());
    case DCOP::setCellText:
      setText(args[0].toInt(), args[1].toInt(), args[2]);
      break;
    case DCOP::insertRow:
      insertRows(args[0].toInt(), args.count() == 1 ? 1 : args[1].toInt());
      break;
    case DCOP::insertColumn:
      insertColumns(args[0].toInt(), args.count() == 1 ? 1 : args[1].toInt());
      break;
    case DCOP::currentColumn:
      return QString::number(currentColumn());
    case DCOP::currentRow:
      return QString::number(currentRow());
    case DCOP::removeColumn:
    {
      int column = args[0].toInt();
      int lines = args[1].toInt();
      for (int i = 0; i < lines; i++)
        removeColumn(column);
      break;
    }  
    case DCOP::removeRow:
    {
      int row = args[0].toInt();
      int lines = args[1].toInt();
      for (int i = 0; i < lines; i++)
        removeRow(row);
      break;
    }
    case DCOP::setColumnCaption:
      horizontalHeader()->setLabel(args[0].toInt(), args[1]);
      break;
    case DCOP::setRowCaption:
      verticalHeader()->setLabel(args[0].toInt(), args[1]);
      break;
    case DCOP::text:
    {
      QString row;
      QString rows;
      for (int r = 0; r < numRows(); r++)
      {
        row = "";
        for (int c = 0; c < numCols(); c++)
        {
          if (c)
            row += "\t";
          row += text(r,c);
        }
        if (r) 
          rows += "\n";
        rows += row;
      }
      return rows;
    }
    case DCOP::setText:
    {
      int r = 0, c = 0;
      setNumCols(0);
      setNumRows(0);
      QStringList rows;
      QStringList row;
      rows = QStringList::split("\n", args[0]);
      setNumRows(rows.count());
      for (QStringList::Iterator it = rows.begin(); it != rows.end(); ++it, ++r) 
      {
        
        row = QStringList::split("\t", *it);
        if (!r)
          setNumCols(row.count());
        c = 0;
        for (QStringList::Iterator itr = row.begin(); itr != row.end(); ++itr, ++c)
          setText(r, c, *itr);
      }
      break;
    }
    /* Specifica */
    case TableFunction::adjustColumn:
    {
    	if (numCols() > args[0].toInt())
    		adjustColumn(args[0].toInt());
    	break;
    }
    case TableFunction::setColumnStretchable:
    {
    	if (numCols() > args[0].toInt())
    		setColumnStretchable(args[0].toInt(), args[1].toInt());
    	break;
    }

    default:
      return KommanderWidget::handleDCOP(function, args);
      
  }  
  return QString::null;
}

/* Table specific */
bool FF_Table::sortWholeRows() const
{
	return doSortWholeRows;
}

void FF_Table::setSortWholeRows(bool newvalue)
{
	doSortWholeRows=newvalue;
}

void FF_Table::sortColumn( int col, bool ascending, bool wholeRows )
{
	wholeRows = sortWholeRows();
	QTable::sortColumn(col, ascending, wholeRows);
	repaintSelections();
}

//#include "kom-fsmi.moc.cpp"
